#!/usr/bin/perl -w

use strict;

use lib ("$ENV{GEMC}/database_io");
use geo;
use geo qw($pi);

my $envelope = 'solid_cherenkov_PVDIS';
my $file     = 'solid_cherenkov_PVDIS.txt';

my $rmin      = 1;
my $rmax      = 1000000;

my %detector = ();    # hash (map) that defines the gemc detector
$detector{"rmin"} = $rmin;
$detector{"rmax"} = $rmax;

use Getopt::Long;
use Math::Trig;

#the virtual volume
sub make_cherenkov_PVDIS
{
 $detector{"name"}        = "cherenkov_PVDIS";
 $detector{"mother"}      = "root";
 $detector{"description"} = "cherenkov_PVDIS";
 $detector{"pos"}         = "0*cm 0*cm 200*cm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "000000";
 $detector{"type"}        = "Tube";
 my $Rin        = 0;
 my $Rout       = 260;
 my $Dz         = 100;
 $detector{"dimensions"}  = "$Rin*cm $Rout*cm $Dz*cm 0*deg 360*deg";
 $detector{"material"}    = "Vacuum";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 0;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "cherenkov_PVDIS";

 print_det(\%detector, $file);
}
make_cherenkov_PVDIS();

# C  ==== Gas Cherenkov definitions
# PARVOL170 'CERV' 840 'SOLE'  0.  0.  200.0   50  'PCON' 15      .0   360.0  4.
#          -0.04 61.0 142.0    24.0  70.3 162.0    24.0  70.3 260.0    90.1  96.0 260.0  

sub make_cherenkov_PVDIS_CERV
{
 $detector{"name"}        = "cherenkov_PVDIS_CERV";
 $detector{"mother"}      = "cherenkov_PVDIS";
 $detector{"description"} = "cherenkov_PVDIS_CERV";
 $detector{"pos"}         = "0*cm 0*cm 0*cm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "E8EA5A";
 $detector{"type"}        = "Polycone";
 my $phiStart    = 0;
 my $phiTotal    = 360;
 my $numZPlanes  = 4;
 $detector{"dimensions"}  = "$phiStart*deg $phiTotal*deg $numZPlanes 61.0*cm 70.3*cm 70.3*cm 96.0*cm 142*cm 162*cm 260*cm 260*cm -0.04*cm 24.0*cm 24.0*cm 90.1*cm";
 $detector{"material"}    = "Air";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "FLUX";
 $detector{"hit_type"}    = "FLUX";
 $detector{"identifiers"} = "cherenkov_PVDIS_CERV";

 print_det(\%detector, $file);
}
make_cherenkov_PVDIS_CERV();

# C  ============== Windows ================== 
# PARVOL171 'CEW1' 209 'CERV'  0.  0. -0.02     0  'TUBE' 3  61.1 141.9 0.02  
# PARVOL172 'CEW2' 209 'CERV'  0.  0. 90.05     0  'TUBE' 3  96.1 260.0 0.05 

sub make_cherenkov_PVDIS_CEW
{
 my $NUM  = 2;
 my @z    = (-0.02,90.05);
 my @Rin  = (61.1,96.1);
 my @Rout = (141.9,260.0);
 my @Dz   = (0.02,0.05);
 my @name = ("CEW1","CEW2"); 
 my @mat  = ("Aluminum","Aluminum");

 for(my $n=1; $n<=$NUM; $n++)
 {
#     my $pnumber     = cnumber($n-1, 10);
    $detector{"name"}        = "cherenkov_PVDIS_$name[$n-1]";
    $detector{"mother"}      = "cherenkov_PVDIS_CERV" ;
    $detector{"description"} = "target_PVDIS_$name[$n-1]";
    $detector{"pos"}        = "0*cm 0*cm $z[$n-1]*cm";
    $detector{"rotation"}   = "0*deg 0*deg 0*deg";
    $detector{"color"}      = "ff0000";
    $detector{"type"}       = "Tube";
    $detector{"dimensions"} = "$Rin[$n-1]*cm $Rout[$n-1]*cm $Dz[$n-1]*cm 0*deg 360*deg";
    $detector{"material"}   = $mat[$n-1];
    $detector{"mfield"}     = "no";
    $detector{"ncopy"}      = $n;
    $detector{"pMany"}       = 1;
    $detector{"exist"}       = 1;
    $detector{"visible"}     = 1;
    $detector{"style"}       = 1;
    $detector{"sensitivity"} = "no";
    $detector{"hit_type"}    = "";
    $detector{"identifiers"} = "target_PVDIS_$name[$n-1]";

    print_det(\%detector, $file);
 }
}
make_cherenkov_PVDIS_CEW();

# C  ============== Volume to be divided =====
# PARVOL173 'CERD' 840 'CERV'  0.  0. 57.0      0  'CONE' 5  33.0  70.3 260.0  96.0 260.0
# C
# C  ============== Divide the barrel ========
# PARVOL174 'CERS'  840 'CERD'  15  0.  0.  2

sub make_cherenkov_PVDIS_CERD
{
 $detector{"name"}        = "cherenkov_PVDIS_CERD";
 $detector{"mother"}      = "cherenkov_PVDIS_CERV";
 $detector{"description"} = "cherenkov_PVDIS_CERD";
 $detector{"pos"}         = "0*cm 0*cm 57*cm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "003333";
 $detector{"type"}        = "Cons";
 my $Rmin1 = 70.3;
 my $Rmax1 = 260.0;
 my $Rmin2 = 96.0;
 my $Rmax2 = 260.0;
 my $Dz    = 33.0;
 my $Sphi  = 0;
 my $Dphi  = 360;
 $detector{"dimensions"}  = "$Rmin1*cm $Rmax1*cm $Rmin2*cm $Rmax2*cm $Dz*cm $Sphi*deg $Dphi*deg";
 $detector{"material"}    = "Air";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 0;
 $detector{"sensitivity"} = "FLUX";
 $detector{"hit_type"}    = "FLUX";
 $detector{"identifiers"} = "cherenkov_PVDIS_CERD";

 print_det(\%detector, $file);
}
make_cherenkov_PVDIS_CERD();

# C  ============== Mirrors definitions ========
# C    f=164.7
# RMATR51    142.6    0.   90.  90.     52.6       0.
# PARVOL175 'CHM1'  860 'CERS'    115.0   0.0 -159.0  -1051  'ELLI'  8  221.5  222.0   33. 70.  155. 205.  0.757 0.757
# PARVOL176 'CHM2'  860 'CERS'      0.0   0.0  -32.7      0  'TUBS'  5  167.0  237.0  0.3  -12.0  12.0 
