#!/usr/bin/perl -w

use strict;

use lib ("$ENV{GEMC}/database_io");
use geo;
use geo qw($pi);

my $envelope = 'solid_solenoid';
my $file     = 'solid_solenoid.txt';

my $rmin      = 1;
my $rmax      = 1000000;

my %detector = ();    # hash (map) that defines the gemc detector
$detector{"rmin"} = $rmin;
$detector{"rmax"} = $rmax;

use Getopt::Long;
use Math::Trig;

#the virtual baffle volume
sub make_solenoid
{
 $detector{"name"}        = "solenoid";
 $detector{"mother"}      = "root";
 $detector{"description"} = "solenoid";
 $detector{"pos"}         = "0*cm 0*cm 0*cm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "ff0000";
 $detector{"type"}        = "Tube";
 my $Rin        = 0;
 my $Rout       = 400;
 my $Dz         = 400;
 $detector{"dimensions"}  = "$Rin*cm $Rout*cm $Dz*cm 0*deg 360*deg";
 $detector{"material"}    = "Vacuum";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 0;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";

 print_det(\%detector, $file);
}
make_solenoid();

# C
# C --     Solenoid coils/yoke
# C --     BaBar NPB 78, 1999: 
# C        Coil mean winding R=153 cm (room temp) ~ 152.2 cm cold
# C        Coil length 351.2 cm room, ~ 350 cm cold
# C        Cryostat: IR=142 cm, OR=177 cm, L=385 cm 
# C
# BaBar Solenoid coils
#===========================================================================================
#GPARVOL30  'SOLE'    0  'HALL'    0.    0.  350.    0  'TUBE'  3  1. 1. 1.
#GPARVOL31  'CRYO'  296  'SOLE'    0.    0.    0.    0  'TUBE'  3  142. 177.  197.5
#GPARVOL32  'CRYV'  203  'CRYO'    0.    0.    0.    0  'TUBE'  3  147. 172.  192.5
#GPARVOL33  'SCL1'  235  'CRYV'    0.    0.    0.    0  'TUBE'  3  152. 154.  175.0

sub make_solenoid_coils_cryo
{
 $detector{"name"}        = "solenoid_coils_cryo";
 $detector{"mother"}      = "solenoid";
 $detector{"description"} = "solenoid_coils_cryo";
 $detector{"pos"}         = "0*cm 0*cm 0*cm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "003333";
 $detector{"type"}        = "Tube";
 my $Rin        = 142;
 my $Rout       = 177;
 my $Dz         = 197.5;
 $detector{"dimensions"}  = "$Rin*cm $Rout*cm $Dz*cm 0*deg 360*deg";
 $detector{"material"}    = "Iron";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";

 print_det(\%detector, $file);
}
make_solenoid_coils_cryo();

sub make_solenoid_coils_cryv
{
 $detector{"name"}        = "solenoid_coils_cryv";
 $detector{"mother"}      = "solenoid_coils_cryo";
 $detector{"description"} = "solenoid_coils_cryv";
 $detector{"pos"}         = "0*cm 0*cm 0*cm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "ff8000";
 $detector{"type"}        = "Tube";
 my $Rin        = 147;
 my $Rout       = 172;
 my $Dz         = 192.5;
 $detector{"dimensions"}  = "$Rin*cm $Rout*cm $Dz*cm 0*deg 360*deg";
 $detector{"material"}    = "Vacuum";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";

 print_det(\%detector, $file);
}
make_solenoid_coils_cryv();

sub make_solenoid_coils_scl1
{
 $detector{"name"}        = "solenoid_coils_scl1";
 $detector{"mother"}      = "solenoid_coils_cryv";
 $detector{"description"} = "solenoid_coils_scl1";
 $detector{"pos"}         = "0*cm 0*cm 0*cm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "ffffff";
 $detector{"type"}        = "Tube";
 my $Rin        = 152;
 my $Rout       = 154;
 my $Dz         = 175;
 $detector{"dimensions"}  = "$Rin*cm $Rout*cm $Dz*cm 0*deg 360*deg";
 $detector{"material"}    = "Aluminum";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";

 print_det(\%detector, $file);
}
make_solenoid_coils_scl1();

#Babar Solenoid barrel yoke simplified
#===========================================================================================
#GPARVOL41  'SY01'  296  'SOLE'    0.    0.    0.    0  'TUBE'  3  180.0 186.0  187.2
#GPARVOL42  'SY02'  296  'SOLE'    0.    0.    0.    0  'TUBE'  3  185.2 189.0  187.2
#GPARVOL43  'SY03'  296  'SOLE'    0.    0.    0.    0  'TUBE'  3  195.6 199.6  187.2
#GPARVOL44  'SY04'  296  'SOLE'    0.    0.    0.    0  'TUBE'  3  206.0 210.0  187.2
#GPARVOL45  'SY05'  296  'SOLE'    0.    0.    0.    0  'TUBE'  3  216.4 220.4  187.2
#GPARVOL46  'SY06'  296  'SOLE'    0.    0.    0.    0  'TUBE'  3  226.8 230.8  187.2
#GPARVOL47  'SY07'  296  'SOLE'    0.    0.    0.    0  'TUBE'  3  237.0 246.0  187.2
#GPARVOL48  'SY08'  296  'SOLE'    0.    0.    0.    0  'TUBE'  3  255.6 264.6  187.2
#GPARVOL49  'SY09'  296  'SOLE'    0.    0.    0.    0  'TUBE'  3  275.0 290.0  187.2

my $color_yoke="F63BFF";

sub make_solenoid_yoke
{
 my $NUM  = 9;
 my @Rin  = (180.0,185.2,195.6,206.0,216.4,226.8,237.0,255.6,275.0);
 my @Rout = (186.0,189.0,199.6,210.0,220.4,230.8,246.0,264.6,290.0);
 my $Dz   = 187.2;

 for(my $n=1; $n<=$NUM; $n++)
 {
#     my $pnumber     = cnumber($n-1, 10);

    $detector{"name"}        = "solenoid_yoke$n";
    $detector{"mother"}      = "solenoid" ;
    $detector{"description"} = "solenoid_yoke$n";
    $detector{"pos"}        = "0*cm 0*cm 0*cm";
    $detector{"rotation"}   = "0*deg 0*deg 0*deg";
    $detector{"color"}      = "$color_yoke";
    $detector{"type"}       = "Tube";
    $detector{"dimensions"} = "$Rin[$n-1]*cm $Rout[$n-1]*cm $Dz*cm 0*deg 360*deg";
    $detector{"material"}   = "Iron";
    $detector{"mfield"}     = "no";
    $detector{"ncopy"}      = $n;
    $detector{"pMany"}       = 1;
    $detector{"exist"}       = 1;
    $detector{"visible"}     = 1;
    $detector{"style"}       = 1;
    $detector{"sensitivity"} = "no";
    $detector{"hit_type"}    = "";
    $detector{"identifiers"} = "solenoid_yoke$n";

    print_det(\%detector, $file);
 }
}
make_solenoid_yoke();

# --    Downstream part        (new)
#GPARVOL51  'SYD1'  296  'SOLE'    0.    0.    0.    0  'PCON'  18 0.  360. 5. 
#            192.5 210.0 240.0  197.5 188.0 240.0  197.5 152.0 240.0  207.0 152.0 240.0  222. 166.0 188.0
#GPARVOL52  'SYD2'  296  'SOLE'    0.    0.    0.    0  'PCON'  15 0.  360. 4. 
#            192.5 240.0 290.0  207.0 240.0 290.0  207.0 257.0 290.0  222.0 265.0 290.0
#GPARVOL53  'SYD3'  296  'SOLE'    0.    0.  331.    0  'TUBE'  3  270.0 290.0   109.0
#GPARVOL54  'SYD4'  296  'SOLE'    0.    0.    0.    0  'PCON'  21 0.  360. 6. 
#            260.0  22.0  70.0  340.0  30.0  94.0  440.0  30.0 118.0  440.0  30.0 290.0  460.0  30.0 290.0  
#            480.0  30.0 120.0

sub make_solenoid_yoke_downstream1
{
 $detector{"name"}        = "solenoid_yoke_downstream1";
 $detector{"mother"}      = "solenoid";
 $detector{"description"} = "solenoid_yoke_downstream1";
 $detector{"pos"}         = "0*cm 0*cm 0*cm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "$color_yoke";
 $detector{"type"}        = "Polycone";
 my $phiStart    = 0;
 my $phiTotal    = 360;
 my $numZPlanes  = 5;
 $detector{"dimensions"}  = "$phiStart*deg $phiTotal*deg $numZPlanes 210.0*cm 188.0*cm 152.0*cm 152.0*cm 166.0*cm 240.0*cm 240.0*cm 240.0*cm 240.0*cm 188.0*cm 192.5*cm 197.5*cm 197.5*cm 207.0*cm 222.0*cm";
 $detector{"material"}    = "Iron";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";

 print_det(\%detector, $file);
}
make_solenoid_yoke_downstream1();

sub make_solenoid_yoke_downstream2
{
 $detector{"name"}        = "solenoid_yoke_downstream2";
 $detector{"mother"}      = "solenoid";
 $detector{"description"} = "solenoid_yoke_downstream2";
 $detector{"pos"}         = "0*cm 0*cm 0*cm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "$color_yoke";
 $detector{"type"}        = "Polycone";
 my $phiStart    = 0;
 my $phiTotal    = 360;
 my $numZPlanes  = 4;
 $detector{"dimensions"}  = "$phiStart*deg $phiTotal*deg $numZPlanes 240.0*cm 240.0*cm 257.0*cm 265.0*cm 290.0*cm 290.0*cm 290.0*cm 290.0*cm 192.5*cm 207.0*cm 207.0*cm 222.0*cm";
 $detector{"material"}    = "Iron";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";

 print_det(\%detector, $file);
}
make_solenoid_yoke_downstream2();

sub make_solenoid_yoke_downstream3
{
 $detector{"name"}        = "solenoid_yoke_downstream3";
 $detector{"mother"}      = "solenoid";
 $detector{"description"} = "solenoid_yoke_downstream3";
 $detector{"pos"}         = "0*cm 0*cm 331*cm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "$color_yoke";
 $detector{"type"}        = "Tube";
 my $Rin        = 270;
 my $Rout       = 290;
 my $Dz         = 109;
 $detector{"dimensions"}  = "$Rin*cm $Rout*cm $Dz*cm 0*deg 360*deg";
 $detector{"material"}    = "Iron";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";

 print_det(\%detector, $file);
}
make_solenoid_yoke_downstream3();

sub make_solenoid_yoke_downstream4
{
 $detector{"name"}        = "solenoid_yoke_downstream4";
 $detector{"mother"}      = "solenoid";
 $detector{"description"} = "solenoid_yoke_downstream4";
 $detector{"pos"}         = "0*cm 0*cm 0*cm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "$color_yoke";
 $detector{"type"}        = "Polycone";
 my $phiStart    = 0;
 my $phiTotal    = 360;
 my $numZPlanes  = 6;
 $detector{"dimensions"}  = "$phiStart*deg $phiTotal*deg $numZPlanes 22.0*cm 30.0*cm 30.0*cm 30.0*cm 30.0*cm 30.0*cm 70.0*cm 94.0*cm 118.0*cm 290.*cm 290.0*cm 120.0*cm 260.0*cm 340.0*cm 440.0*cm 440.0*cm 460.0*cm 480.0*cm";
 $detector{"material"}    = "Iron";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";

 print_det(\%detector, $file);
}
make_solenoid_yoke_downstream4();

# --    Upstream   part        (new)
#GPARVOL61  'SYU1'  296  'SOLE'    0.    0.    0.    0  'PCON'  18 0.  360. 5. 
#           -192.5 210.0 240.0 -197.5 188.0 240.0 -197.5 152.0 240.0 -207.0 152.0 240.0 -222. 166.0 188.0
#GPARVOL62  'SYU2'  296  'SOLE'    0.    0.    0.    0  'PCON'  15 0.  360. 4. 
#           -192.5 240.0 290.0 -207.0 240.0 290.0 -207.0 257.0 290.0 -222.0 265.0 290.0
#GPARVOL63  'SYU3'  296  'SOLE'    0.    0. -224.25  0  'TUBE'   3    265.0 290.0  2.25
#GPARVOL64  'SYU4'  296  'SOLE'    0.    0. -244.5   0  'CONE'   5  18.0  41.  290.   57.5 290.
#GPARVOL65  'SYU5'  296  'SOLE'    0.    0. -271.5   0  'CONE'   5   5.0  34.0 290.  39.0 290.
#GPARVOL66  'SYU6'  296  'SOLE'    0.    0. -280.5   0  'CONE'   5   2.0  31.5 290.  33.5 290.

sub make_solenoid_yoke_upstream1
{
 $detector{"name"}        = "solenoid_yoke_upstream1";
 $detector{"mother"}      = "solenoid";
 $detector{"description"} = "solenoid_yoke_upstream1";
 $detector{"pos"}         = "0*cm 0*cm 0*cm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "$color_yoke";
 $detector{"type"}        = "Polycone";
 my $phiStart    = 0;
 my $phiTotal    = 360;
 my $numZPlanes  = 5;
 $detector{"dimensions"}  = "$phiStart*deg $phiTotal*deg $numZPlanes 210.0*cm 188.0*cm 152.0*cm 152.0*cm 166.0*cm 240.0*cm 240.0*cm 240.0*cm 240.0*cm 188.0*cm -192.5*cm -197.5*cm -197.5*cm -207.0*cm -222.0*cm";
 $detector{"material"}    = "Iron";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";

 print_det(\%detector, $file);
}
make_solenoid_yoke_upstream1();

sub make_solenoid_yoke_upstream2
{
 $detector{"name"}        = "solenoid_yoke_upstream2";
 $detector{"mother"}      = "solenoid";
 $detector{"description"} = "solenoid_yoke_upstream2";
 $detector{"pos"}         = "0*cm 0*cm 0*cm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "$color_yoke";
 $detector{"type"}        = "Polycone";
 my $phiStart    = 0;
 my $phiTotal    = 360;
 my $numZPlanes  = 4;
 $detector{"dimensions"}  = "$phiStart*deg $phiTotal*deg $numZPlanes 240.0*cm 240.0*cm 257.0*cm 265.0*cm 290.0*cm 290.0*cm 290.0*cm 290.0*cm -192.5*cm -207.0*cm -207.0*cm -222.0*cm";
 $detector{"material"}    = "Iron";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";

 print_det(\%detector, $file);
}
make_solenoid_yoke_upstream2();

sub make_solenoid_yoke_upstream3
{
 $detector{"name"}        = "solenoid_yoke_upstream3";
 $detector{"mother"}      = "solenoid";
 $detector{"description"} = "solenoid_yoke_upstream3";
 $detector{"pos"}         = "0*cm 0*cm -224.25*cm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "$color_yoke";
 $detector{"type"}        = "Tube";
 my $Rin        = 265.0;
 my $Rout       = 290.0;
 my $Dz         = 2.25;
 $detector{"dimensions"}  = "$Rin*cm $Rout*cm $Dz*cm 0*deg 360*deg";
 $detector{"material"}    = "Iron";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";

 print_det(\%detector, $file);
}
make_solenoid_yoke_upstream3();

sub make_solenoid_yoke_upstream4
{
 $detector{"name"}        = "solenoid_yoke_upstream4";
 $detector{"mother"}      = "solenoid";
 $detector{"description"} = "solenoid_yoke_upstream4";
 $detector{"pos"}         = "0*cm 0*cm -244.5*cm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "$color_yoke";
 $detector{"type"}        = "Cons";
 my $Rmin1 = 41.0;
 my $Rmax1 = 290.0;
 my $Rmin2 = 57.5;
 my $Rmax2 = 290.0;
 my $Dz    = 18.0;
 my $Sphi  = 0;
 my $Dphi  = 360;
 $detector{"dimensions"}  = "$Rmin1*cm $Rmax1*cm $Rmin2*cm $Rmax2*cm $Dz*cm $Sphi*deg $Dphi*deg";
 $detector{"material"}    = "Iron";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";

 print_det(\%detector, $file);
}
make_solenoid_yoke_upstream4();

sub make_solenoid_yoke_upstream5
{
 $detector{"name"}        = "solenoid_yoke_upstream5";
 $detector{"mother"}      = "solenoid";
 $detector{"description"} = "solenoid_yoke_upstream5";
 $detector{"pos"}         = "0*cm 0*cm -271.5*cm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "$color_yoke";
 $detector{"type"}        = "Cons";
 my $Rmin1 = 34.0;
 my $Rmax1 = 290.0;
 my $Rmin2 = 39.0;
 my $Rmax2 = 290.0;
 my $Dz    = 5.0;
 my $Sphi  = 0;
 my $Dphi  = 360;
 $detector{"dimensions"}  = "$Rmin1*cm $Rmax1*cm $Rmin2*cm $Rmax2*cm $Dz*cm $Sphi*deg $Dphi*deg";
 $detector{"material"}    = "Iron";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";

 print_det(\%detector, $file);
}
make_solenoid_yoke_upstream5();

sub make_solenoid_yoke_upstream6
{
 $detector{"name"}        = "solenoid_yoke_upstream6";
 $detector{"mother"}      = "solenoid";
 $detector{"description"} = "solenoid_yoke_upstream6";
 $detector{"pos"}         = "0*cm 0*cm -280.5*cm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "$color_yoke";
 $detector{"type"}        = "Cons";
 my $Rmin1 = 31.5;
 my $Rmax1 = 290.0;
 my $Rmin2 = 33.5;
 my $Rmax2 = 290.0;
 my $Dz    = 2.0;
 my $Sphi  = 0;
 my $Dphi  = 360;
 $detector{"dimensions"}  = "$Rmin1*cm $Rmax1*cm $Rmin2*cm $Rmax2*cm $Dz*cm $Sphi*deg $Dphi*deg";
 $detector{"material"}    = "Iron";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 1;
 $detector{"style"}       = 1;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";

 print_det(\%detector, $file);
}
make_solenoid_yoke_upstream6();
#===========================================================================================


